      program SCATTR
cc
c   SCATTR: provide run parameters for subroutine SCATT2.
c           For description of the run parameters, see first set of
c              comment lines in SCATT2.
c           This version is self-contained and doesn't require that
c              agramlib exists.  This limited version requires that
c              the input file be named SCATTR.IN and the output file
c              be named SCATTR.OUT.  For an alternative version that
c              does reference agramlib but also gets its input and
c              output file names from the command line, see
c              CLISCA.FOR.  CLISCA and this subroutine both call
c              subroutine SCATT2, below.
c           To find machine-dependent code, locate lines that begin
c              with "c ### ".
cc
      character*10 infile,tocfil
      character*7 cmputr
      character*4 ccc
      logical upper, includ
cc
      parameter (infile = 'SCATTR.IN', tocfil='SCATTR.OUT')
c ### for PC with MS fortran:    parameter (cmputr = 'PC/DOS')
c ### for VAX:
      parameter (cmputr = 'VAX/VMS')
      parameter (ccc    = 'NONE' )
      parameter (upper=.false., includ = .false.)
cc
      call SCATT2 (infile,tocfil, cmputr, ccc, upper, includ)
      stop
cccccccccccccccccccc (end of SCATTR) ccccccccccccccccccccccccccccccccccc
      end
      subroutine scaopn (lun, fname,newold,ierr)
      character*(*) fname, newold
cc
c   SCAOPN contains non-standard open statements required in SCATT2.
cc
      if (newold.eq.'OLD' .or. newold.eq.'old') then
         open (unit=lun, file=fname, status='old',iostat=ierr
c ### non standard VAX fortran:
     x,         READONLY)
c ### non standard MS fortran:     x,        mode='READ')
c ### standard fortran:          x        )
      else
         open  (unit=lun, file=fname, iostat=ierr
c ### VAX fortran:
     x,         status='new', CARRIAGECONTROL = 'LIST')
c ###  MS fortran:     x,       status='unknown', mode='WRITE')
c ### standard fortran:   x,       status='new')
      endif
      return
cccccccccccccccccccc (end of SCAOPN/SCATT2) cccccccccccccccccccccccccccc
      end
      subroutine SCATT2 (infile,tocfil, cmputr, ccc, upper, includ)
      character*(*) infile,tocfil,cmputr,ccc
      logical upper, includ
cc
c   SCATT2:  read an input file (usually named SCATTR.IN), write all
c            the output files identified in the input file (SCATTR.IN)
c            plus an extra output file (usually named SCATTR.OUT) that
c            contains a list of all the output file names.
c
c   SCATTR.IN consists of a concatenated series of subfiles, each of
c             which will be written to a separate output file by this
c             program.  Each subfile in SCATTR.IN is prefaced with a
c             line having a carat (^) as the first character, the
c             subfile name as the last few characters, and a series of
c             dots between the carat and the subfile name.  Each
c             subfile is followed by a line that contains a single
c             carat in the first column.
c
c   SCATTR.IN is created as GATHER.OUT by the GATHER program or is
c             created with any name by the WORKSHOP program.
cc
c   On entry --
c     INFILE  = name of the input file.  (usually SCATTR.IN)
c     TOCFIL  = name of the output file that will contain a list of the
c                 names of all the other output files.
c                 (usually SCATTR.OUT)
c     CMPUTR  = 'VAX/VMS' or 'PC/DOS' (and others may be added later)
c                 to identify the local computer.
c     CCC     = 'NONE' (and maybe later 'A2E' or 'E2A') to indicate
c                  whether the character codes should be converted
c                  from ASCII to EBCIDIC.
c     UPPER   = .true. if all characters in the output files should
c                  be in upper case
c             = .false. if case in output files should remain the same
c                  as in the input file.
c     INCLUD  = .true. if include statements should be replaced by
c                  the contents of the indicated file.
c             = .false. if not.
c
cc
c   Translations:
c     The source computer is indicated on the first line of the
c     input file as 'PC/DOS', 'VAX/VMS', or 'EDT' (EDT is
c     equivalent to 'VAX/VMS').  This identifier is compared to
c     CMPUTR to determine whether any of the lines from the input
c     file should be translated before writing them to the output
c     file, consistent with the different conventions used on the
c     source and local fortan compilers.  Only those translations
c     that apply to coding conventions used in the AGRAM code are
c     applied; SCATTR is NOT a general-purpose translator.
c     When:
c     Source id = CMPUTR, then no translations are performed other
c        than the conversions indicated by CCC, UPPER, and INCLUD.
c     Source id = 'VAX/VMS' and CMPUTR= 'PC/DOS', then SCATTR will
c        translate as though the input file were appropriate for the
c        VAX/VMS fortran compiler and the output files appropriate
c        for the MicroSoft fortran compiler on an IBM (or clone) AT
c        machine.
c        a) output file names may be truncated and the suffixes may
c           be renamed.
c          - VAX file names may be any length, DOS file names must
c            be 8 or fewer characters in the name itself, 3 or
c            fewer characters in the extension.
c          - VAX file name suffixes will be changed as follows:
c             .COM becomes .VXC (for vax com)
c             .BAT         .VXB
c             .LNK         .VXL
c             .MAK         .VXM
c             .OUT         .VXO
c             .WRK         .VXW
c        b) include statements
c           - The VAX/VMS fortran compiler's include statements take
c             the form: INCLUDE '<filename>'
c                       INCULDE '<filename>/NOLIST'
c                    or INCLUDE '<filename>/LIST'
c              with the INCLUDE beginning in column 7 or greater.
c           - The Microsoft fortran compilter for AT/DOS machines
c             accepts include statements in the first VAX/VMS form:
c                       INCLUDE '<filename>'
c             so if /NOLIST or /LIST is in an input include statement
c             it is removed before the statement is written to the
c             output file.
c        c) SAVE statements are commented out.  The 5.0 version of the
c           MS fortran compiler complains if there is more than one
c           subroutine in a file that has the same SAVE statement.
c           Since SAVEs aren't needed with this compiler anyway, we'll
c           just comment them out here with: "c ### "
c     Source id = 'PC/DOS' and CMPUTR= 'VAX/VMS', then SCATTR will
c        translate as though the input file were for the Microsoft
c        PC fortran compiler and the output files for the VAX/VMS
c        fortran compiler.
c        a) output file name extensions may be be renamed:
c             .BAT becomes .PCB
c             .MAK         .PCM
c             .INC         .PCI
c             .OUT         .PCO
c             .WRK         .PCW
c        b) INCLUDE statements are unchanged.
c        c) Remove any leading "c ###" comments from SAVE statements.
cc
      logical newnam
      character*132 line, fname
      character*132 fixc
      character*1 carat
      parameter (carat='^')
      parameter (lunmsg=6, lunlst=3, lunin=1, lunout=2)
      parameter (kv2p = 2, kp2v = 1)
cc
      konvrt = 0
      nfiles = 0
      nline  = 0
      nchar = lnbc(tocfil)
      n     = lnbc(infile)
      call upcase(tocfil(1:nchar))
      call upcase(infile(1:n))
      n = min (n, 130-12-47)
      write (lunmsg,1000) infile(1:n), tocfil(1:nchar), tocfil(1:nchar)
cc
c   Check that the run parameters given are recognizable
cc
      if ( cmputr.ne. 'VAX/VMS' .and. cmputr.ne.'PC/DOS') then
         write (lunmsg,3005) 'CMPUTR', cmputr
         goto 9000
      endif
      if (ccc.ne.'NONE') then
         write (lunmsg,3005) 'CCC', ccc
         goto 9000
      endif
      if (includ) then
         write (lunmsg, 3004)
         goto 9000
      endif
cc
c   Open the list file
cc
      call scaopn(lunlst,tocfil(1:nchar),'new',ierr)
      if (ierr.ne.0) then
         write(lunmsg,3001) tocfil(1:nchar)
         go to 9000
      endif
      write (lunlst,2001) cmputr, tocfil(1:nchar)
      write (lunmsg,1001)
      write (lunmsg,1012) tocfil(1:nchar)
cc
c   Open the input file
cc
      call scaopn(lunin,infile(1:n),'old',ierr)
      if (ierr.ne.0) then
         write(lunmsg,3001) infile(1:n)
         write(lunmsg,3002)
         go to 9000
      endif
cc
c   Read the first two lines of the input file to see whether we're
c     supposed to be converting from one compiler format to another,
c     then reset KONVRT accordingly.
c   The first line of the first file in a gathered collection is very
c     likely a .WRK file that has "PC/DOS", "VAX/VMS", or "EDT" on the
c     first line.  If not, just assume that the source computer is the
c     same as the local computer.  Note that "VAX/VMS" and "EDT" both
c     indicate that we have a gathered collection of VAX/VMS files.
cc
   20 continue
      read (lunin, '(a)') line
      if (line(1:1).eq. carat) goto 20
      nchar = lnbc(line)
      if (nchar.lt.1) goto 20
      fixc = line
      call upcase(fixc)
      loc1 = index(fixc,'*')
      if (loc1.eq.0) loc1 = 1
      if (0.ne.index(fixc(loc1:),'EDT ') .and.
     x         cmputr.eq.'PC/DOS') then
         konvrt=kv2p
         write (lunmsg,1013) 'VAX/VMS',cmputr
      else if (0.ne.index(fixc(loc1:),'VAX/VMS ') .and.
     x         cmputr.eq.'PC/DOS') then
         konvrt=kv2p
         write (lunmsg,1013) 'VAX/VMS', cmputr
      elseif (0.ne.index(fixc(loc1:),'PC/DOS ') .and.
     x         cmputr.eq.'VAX/VMS') then
         konvrt=kp2v
         write (lunmsg,1013) 'PC/DOS', cmputr
      endif
      rewind (unit=lunin)
cc
c  Loop for each subfile named in the input file.
c  a) read the line containing output file name:
cc
  100 continue
      read(lunin,'(a)',end=9000) line
      nline = nline + 1
      nchar = lnbc(line)
      if (nchar.lt.1) goto 100
      if (nchar.le.1 .or. (line(1:1) .ne. carat)) goto 8000
      do 102 i = 2,nchar
      if(line(i:i) .ne. '.')then
         fname = line(i:nchar)
         nchar = nchar - i + 1
         call upcase (fname)
         goto 103
      endif
  102 continue
      goto 8000
  103 continue
cc
c   b) translate the output file name from VAX style to DOS style,
c      if requested.
cc
      newnam = .false.
      if (konvrt.eq. kv2p) then
         call VAXDOS(fixc, fname, newnam, ierr)
         if (ierr.ne.0) goto 8000
      elseif (konvrt.eq. kp2v) then
         call DOSVAX(fixc, fname, newnam, ierr)
         if (ierr.ne.0) goto 8000
      endif
cc
c   c) Open the output file
cc
      nfiles = nfiles + 1
      if (newnam) then
         write (lunmsg,1011) nfiles, fname(1:nchar), fixc(1:lnbc(fixc))
         fname = fixc
         nchar = lnbc(fname)
      else
         write (lunmsg,1010) nfiles, fname(1:nchar)
      endif
      write (lunlst,'(a)') fname(1:nchar)
      call scaopn (lunout, fname(1:nchar), 'new', ierr)
      if (ierr.ne.0) then
         write(lunmsg,3001) fname(1:nchar)
         go to 9000
      endif
cc
c   d) Copy lines from input to output file:
cc
  110 continue
      read (lunin, '(a)') line
      if (line(1:1).ne. carat) then
         nchar = lnbc(line)
         if (nchar.lt.1) then
            nchar = 1
            line = ' '
         else
            nline = nline + 1
cc
c   d.1) character code conversion
cc
            if (ccc.ne.'NONE') then
               write (lunmsg,3005) 'CCC', ccc
               goto 9000
c              +++
c              add character code conversion here later, if it's needed.
c              +++
            endif
cc
c   d.2) include file inclusion
cc
            if (includ) then
               write (lunmsg, 3004)
               goto 9000
c              +++
c              Add include code here later, if it's ever needed.
c              Remember:
c              - do the charactercode conversion on the included stuff
c                too
c              - allow nested includes
c              - require and check that an included subfile has been
c                given in the input file before any include statment
c                that references it.
c              +++
            endif
cc
c   d.3) PC/DOS to VAX/VMS  SAVE statement conversion:
cc
            if (konvrt.eq.kp2v) then
               if (line(3:5).eq.'###') then
                  fixc = line(1:11)
                  call upcase(fixc(1:11))
                  if (fixc(1:11).eq.'C ### SAVE ')
     x               line(1:11) = '      save '
               endif
cc
c   d.4) VAX/VMS to PC/DOS conversions:
c
c   d.4.1) SAVE statement conversion:
cc
            else if (konvrt.eq.kv2p) then
               loc1 = nbc(line)
               if (loc1.gt.6 .and. (line(loc1:loc1+4).eq.'SAVE ' .or.
     x                              line(loc1:loc1+4).eq.'save ')) then
                   if (0.eq.index(line(loc1+5:), '='))
     x               line(1:loc1+4) = 'c ### save '
               endif
cc
c   d.4.2) INCLUDE statement conversion:
cc
               loc1 = index(line,'include')
               if (loc1.eq.0 ) loc1 = index(line,'INCLUDE')
               loc2 = nbc  (line)
               if (loc1.eq.loc2 .and. loc1.ge.7
     x             .and. line(nchar:nchar).eq.'''') then
                  loc1 = index(line, '''')
                  loc2 = index(line, '/')
                  if (loc2.eq.0) then
                     loc2 = nchar
                     if (line(loc2:loc2) .ne. '''') goto 8000
                  endif
                  if (loc1.eq.0) goto 8000
                  call VAXDOS(fixc,line(loc1+1:loc2-1),newnam, ierr)
                  if (ierr.eq.0) then
                     if (newnam) then
                        nchar = lnbc(fixc)
cwas: %%%               line  = '$INCLUDE:'''//fixc(1:nchar)//''''
                        line(loc1+1:) = fixc(1:nchar)//''''
                     else
                        line(loc2:) = ''''
                     endif
                     nchar = lnbc(line)
                  else if (ierr.ne.2) then
                     goto 8000
                  endif
               endif
            endif
         endif
cc
c   d.5) upper case conversion
cc
            if (upper) call upcase (line)
cc
c   d.6) write current line to current output file:
cc
         write(lunout, '(a)') line(1:nchar)
         goto 110
      endif
cc
c   e) close the current output file
c      then loop back to treat the next output file.
cc
      close (unit = lunout)
      go to 100
cc
c   Error in input file:
cc
 8000 continue
      write(lunmsg,3003) infile, nline, line(1:max(1,lnbc(line)))
cc
c   Done
cc
 9000 continue
      write (lunmsg,'(1h/)')
      stop
cc
c   Formats
cc
 1000 format(/'     SCATTR '
     x,    'will scatter the concatinated subfiles in the file named'
     x/12x, a,', writing each to a separate file.  SCATTR will'
     x/12x,'also generate a file named ',a,' that lists the names'
     x/12x,'of the other files generated.  ',a,' can be used as'
     x/12x,'input to the GATHER program to recollect a copy of all'
     x/12x,'the separate files into another single file.')
cwas: 1001 format(
cwas:     x/ 9x, '   input                  altered'
cwas:     x/ 9x, 'subfile name          output file name'
cwas:     x/ 9x, '============          ================')
 1001 format(
     x/ 9x, '   Input              Output file name'
     x/ 9x, 'subfile name          if not= input name'
     x/ 9x, '============          ==================')
 1002 format(/5x, 'Output files:' /)
 1010 format(2x, i5,': ', a)
 1011 format(2x, i5,': ', a, t32, a)
 1012 format( t32, '(',a, ')')
 1013 format(/5x, 'Converting from ',a,' to ',a,' format.' /)
cc
 2001 format( 4x, a,
     x/';;'/';  This is ', a, ' = output from the SCATTR program'
     x/';;')
cc
 3001 format(/' *** Error opening File = ', a)
 3002 format(5x, 'Is it missing?  Ending Run. ***')
 3003 format(/
     x ' *** Input file (=', a, ') format error in line', i6, ':'
     x/1x,a, /5x,'Ending run. ***')
 3004 format (/' *** '
     x, 'Sorry, but code for INCLUD = .TRUE. has not been coded yet.'
     x/5x, 'Ending run. ***')
 3005 format (/' *** '
     x, 'Sorry, but the FORTRAN for ', a, ' = ', a,
     x  ' has not been coded (yet).'
     x/5x, 'Ending run. ***')
ccccccccccccccccc (end of SCATT2) cccccccccccccccccccccccccccccccccccccc
      end
      subroutine VAXDOS(fixc, fname, newnam, ierr)
      logical newnam
      character*(*) fixc, fname
cc
c   VAXDOS: convert from a VAX style file name to a DOS style name.
c
c   On entry:
c     FNAME  = a VAX file name
c   On return --
c     IERR   = 0 if all is OK
c              1 if serious error
c              2 if trying to convert a VAX-dependent include statement
c                   like INCLUDE '($SSDEF)'.  Just don't bother with it,
c                   since the whole subroutine is VAX-dependent.
c     fixc = a valid DOS file name (if IERR=0)
c     NEWNAM = .true. if FNAME wasn't a valid DOS file name
cc
      character*3 suffix
cc
      ierr   = 0
      newnam = .false.
      fixc = fname
      locdot = index(fname,'.')
      if (locdot.le.0) then
         ierr = 1
         loc1 = index(fname,'(')
         loc2 = index(fname,')')
         if (loc1.lt.loc2 .and. loc1.lt.index(fname,'$')) ierr = 2
         return
      endif
      loc1   = locdot + 1
      loc2   = lnbc(fname)
      if (locdot.gt.9) then
          newnam = .true.
          locdot = 9
          fixc(locdot:locdot) = '.'
      endif
      suffix = fname(loc1:loc2)
      if (suffix.eq.'COM') suffix = 'VXC'
      if (suffix.eq.'BAT') suffix = 'VXB'
      if (suffix.eq.'LNK') suffix = 'VXL'
      if (suffix.eq.'MAK') suffix = 'VXM'
      if (suffix.eq.'OUT') suffix = 'VXO'
      if (suffix.eq.'WRK') suffix = 'VXW'
      fixc(locdot+1:) = suffix
      if (suffix .ne. fname(loc1:loc2)) newnam = .true.
      return
cccccccccccccccccccc (end of VAXDOS/SCATT2) cccccccccccccccccccccccccccc
      end
      subroutine DOSVAX(fixc, fname, newnam, ierr)
      logical newnam
      character*(*) fixc, fname
cc
c   DOSVAX: convert from a PC/DOS style file name to a VAX/VMS style
c           file name.
c
c   On entry:
c     FNAME  = a PC/DOS file name
c   On return --
c     IERR   = 0 if all is OK
c              1 if error
c     FIXC   = a valid DOS file name (if IERR=0)
c     NEWNAM = .true. if FNAME wasn't a valid DOS file name
cc
      character*3 suffix
cc
      ierr   = 0
      newnam = .false.
      fixc = fname
      locdot = index(fname,'.')
      if (locdot.le.0) then
         ierr = 1
         return
      endif
      loc1   = locdot + 1
      loc2   = lnbc(fname)
      suffix = fname(loc1:loc2)
      if (suffix.eq.'BAT') suffix = 'PCB'
      if (suffix.eq.'MAK') suffix = 'PCM'
      if (suffix.eq.'INC') suffix = 'PCI'
      if (suffix.eq.'OUT') suffix = 'PCO'
      if (suffix.eq.'WRK') suffix = 'PCW'
      fixc(locdot+1:) = suffix
      if (suffix .ne. fname(loc1:loc2)) newnam = .true.
      return
cccccccccccccccccccc (end of DOSVAX/SCATT2) cccccccccccccccccccccccccccc
      end
c ======================================================================
c
c   The following three subroutines, LNBC, NBC, and UPCASE, were copied
c   from CSTRING.FOR in AGRAMLIB on 15mar89 so the SCATTR.FOR file
c   could be a complete, self-contained program.
c
c ======================================================================
      integer function lnbc(cs)
      character*(*) cs
cc
c   Find the last non-blank character
cc
c +++ include 'charcodes.inc/nolist' +++ define knull here:
      parameter (knull =0)
cc
      n=len(cs)
      do 10 k=n,1,-1
      if(cs(k:k).ne.' ' .and. ichar(cs(k:k)).ne.knull) then
         lnbc=k
         go to 11
      endif
   10 continue
      lnbc=0
   11 continue
      return
cccccccccccccccccccc (end of lnbc/SCATT2) ccccccccccccccccccccccccccccc
      end
      integer function nbc(cs)
      character*(*) cs
cc
c   Find the first non-blank character in a string.
cc
c +++ include 'charcodes.inc/nolist' +++ define knull here:
      parameter (knull =0)
cc
      n=len(cs)
      do 10 k=1,n
      if(cs(k:k).ne.' ' .and. ichar(cs(k:k)).ne.knull) then
         nbc=k
         go to 11
      endif
   10 continue
      nbc=0
   11 continue
      return
cccccccccccccccccccc (end of nbc) ccccccccccccccccccccccccccccccccccc
      end
      subroutine upcase(cs)
      character*(*) cs
      character*1 c
cc
c   Convert any lower-case characters in CS to upper case.
cc
      n=len(cs)
cc
      do 10 k=1,n
      c= cs(k:k)
      if(c.eq.' ') go to 10
      if(llt(c,'a') .or. lgt(c,'z')) go to 10
      if(c.eq.'a') c='A'
      if(c.eq.'b') c='B'
      if(c.eq.'c') c='C'
      if(c.eq.'d') c='D'
      if(c.eq.'e') c='E'
      if(c.eq.'f') c='F'
      if(c.eq.'g') c='G'
      if(c.eq.'h') c='H'
      if(c.eq.'i') c='I'
      if(c.eq.'j') c='J'
      if(c.eq.'k') c='K'
      if(c.eq.'l') c='L'
      if(c.eq.'m') c='M'
      if(c.eq.'n') c='N'
      if(c.eq.'o') c='O'
      if(c.eq.'p') c='P'
      if(c.eq.'q') c='Q'
      if(c.eq.'r') c='R'
      if(c.eq.'s') c='S'
      if(c.eq.'t') c='T'
      if(c.eq.'u') c='U'
      if(c.eq.'v') c='V'
      if(c.eq.'w') c='W'
      if(c.eq.'x') c='X'
      if(c.eq.'y') c='Y'
      if(c.eq.'z') c='Z'
      cs(k:k) =c
   10 continue
      return
cccccccccccccccccccc (end of upcase/SCATT2) cccccccccccccccccccccccccccc
      end
 
